#include "wallwindow.h"

int switch_time=60, row_num=0, number;

WallWindow::WallWindow()
{
    createLayout();

    settings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
                             "wallpapers-changer", "wallchanger");

    //Timer
    switchTimer = new QTimer(this);
    connect(switchTimer, SIGNAL(timeout()), this, SLOT(set_image()));

    connect(closeButton,SIGNAL(clicked()),this,SLOT(quitButton_clicked()));
    connect(startButton,SIGNAL(clicked()),this,SLOT(start_clicked()));
    connect(clearButton,SIGNAL(clicked()),this,SLOT(clear_clicked()));
    connect(addButton,SIGNAL(clicked()),this,SLOT(add_clicked()));

    connect(onlynames_checkBox,SIGNAL(stateChanged(int)),this,
            SLOT(on_onlynames_checkBox_stateChanged(int)));
    connect(timeBox,SIGNAL(valueChanged(int)),this,SLOT(on_timeBox_valueChanged(int)));
    connect(tableWidget,SIGNAL(cellDoubleClicked(int,int)),this,
            SLOT(on_tableWidget_cellDoubleClicked(int,int)));
    connect(pos_comboBox,SIGNAL(currentIndexChanged(QString)),this,
            SLOT(on_pos_comboBox_currentIndexChanged(QString)));
    connect(order_comboBox,SIGNAL(currentIndexChanged(QString)),this,
            SLOT(order_comboBox_currentIndexChanged(QString)));

    connect(tableWidget,SIGNAL(cellClicked(int,int)),this,
            SLOT(on_tableWidget_cellClicked(int,int)));

    //Tray menu
    QAction* restoreAction = new QAction(tr("&Restore"), this);
    connect(restoreAction, SIGNAL(triggered()), this, SLOT(showNormal()));
    QAction* quitAction = new QAction(tr("&Quit"), this);
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
    QMenu* trayIconMenu = new QMenu(this);
    trayIconMenu->addAction(restoreAction);
    trayIconMenu->addAction(quitAction);
    delicon = new QIcon("/usr/share/icons/gnome/48x48/actions/edit-delete.png");

    if (settings->value("settings/show_only_names")==true)
        onlynames_checkBox->setCheckState(Qt::Checked);
    timeBox->setValue(settings->value("settings/scroll_time", 10).toInt());
    scroll_label->setText(settings->value("settings/in", "Scroll time(minutes):").toString());
    pos_comboBox->setCurrentIndex(settings->value("settings/img_position", 0).toInt());
    order_comboBox->setCurrentIndex(settings->value("settings/img_order",0).toInt());

    if (scroll_label->text().contains("hour"))
    {
        timeBox->setMaximum(24);
        timeBox->setMinimum(0);
    }

    //Set tray icon
    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setContextMenu(trayIconMenu);
    trayicon = new QIcon("/usr/share/icons/hicolor/48x48/apps/wallpaper-changer.png");
    trayIcon->setIcon(*trayicon);
    trayIcon->show();

    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
    this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));

    setWindowTitle(tr("Wallpapers Changer 0.1"));
    setWindowIcon(*trayicon);

    allFiles=load_and_show();
    rand_numbers=create_randlist();
    if (allFiles.count()!=0) start_clicked();
}

void WallWindow::createLayout()
{
    imgpos_label = new QLabel("Image position:");
    countlabel = new QLabel("Image counts:");
    scroll_label = new QLabel("Scroll time(minutes):");
    scroll_label->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

    startButton = new QPushButton(tr("Start"));
    closeButton = new QPushButton(tr("Quit"));
    clearButton = new QPushButton(tr("Clear"));
    addButton = new QPushButton(tr("Add"));

    //Table configure
    tableWidget = new QTableWidget;
    tableWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    tableWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    tableWidget->setAutoScroll(false);
    tableWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
    tableWidget->setSelectionMode(QAbstractItemView::NoSelection);
    tableWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
    tableWidget->setShowGrid(false);
    tableWidget->horizontalHeader()->setVisible(false);
    tableWidget->verticalHeader()->setVisible(false);

    QStringList poslist, img_orfer_list;
    poslist<<"None"<<"Wallpaper"<<"Centered"<<"Scaled"<<"Stretched"<<"Zoom"<<"Spanned";
    img_orfer_list<<"Linear"<<"Shuffle";

    timeBox = new QSpinBox;
    pos_comboBox = new QComboBox;
    pos_comboBox->addItems(poslist);
    onlynames_checkBox = new QCheckBox(tr("Only names"));
    img_order_label = new QLabel(tr("Order of showing image:"));
    order_comboBox = new QComboBox;
    order_comboBox->addItems(img_orfer_list);

    QGridLayout *layout = new QGridLayout;

    layout->addWidget(scroll_label,0,0);
    layout->addWidget(timeBox,0,6,Qt::AlignRight);

    layout->addWidget(countlabel,1,0);
    layout->addWidget(onlynames_checkBox,1,6,Qt::AlignRight);

    layout->addWidget(tableWidget,2,0,1,0);

    layout->addWidget(imgpos_label,3,0);
    layout->addWidget(pos_comboBox,3,6,Qt::AlignRight);

    layout->addWidget(img_order_label,4,0,1,0);
    layout->addWidget(order_comboBox,4,6,Qt::AlignRight);

    layout->addWidget(clearButton,5,0);
    layout->addWidget(addButton,5,4);
    layout->addWidget(startButton,5,5);
    layout->addWidget(closeButton,5,6);

    setMinimumHeight(300);
    resize(400,500);
    setLayout(layout);
}

void WallWindow::tableFill()
{
    tableWidget->setRowCount(allFiles.count());
    tableWidget->setColumnCount(2);
    QPixmap delIcon;
    delIcon.load("/usr/share/icons/gnome/48x48/actions/edit-delete.png");

    for(int i=0; i<allFiles.count(); i++)
    {
        removeLabel.append(new QLabel(this));
        //removeButtons.append(createButton("Del", SLOT(digitClicked()),i));

        removeLabel.at(i)->setPixmap(delIcon.scaledToHeight(30,Qt::SmoothTransformation));
        removeLabel.at(i)->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
        //removeButtons[i]->setIcon(*delicon);
        //removeButtons[i]->setAutoRaise(true);
        //removeButtons[i]->setToolTip("Delete from list");
        //removeButtons[i]->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
        //tableWidget->setCellWidget(i, 0, removeButtons[i]);
        tableWidget->setCellWidget(i, 0, removeLabel[i]);

        QTableWidgetItem* Item = new QTableWidgetItem();
        if (onlynames_checkBox->isChecked())
            Item->setText(QString(allFiles.at(i).fileName()));
        else
            Item->setText(QString(allFiles.at(i).absoluteFilePath()));
        tableWidget->setItem(i, 1, Item);
    }
    countlabel->setText("Images count: "+QString::number(allFiles.count()));
    tableWidget->resizeColumnsToContents();
    tableWidget->resizeColumnsToContents();
}


QFileInfoList WallWindow::load_and_show()
{
    QStringList files = settings->value("settings/file_list").toStringList();
    for (int i=0; i<files.count(); i++)
    {
        QFileInfo info(files[i]);
        allFiles.append(info);
    }
    tableFill();
    return allFiles;
}

void WallWindow::start_clicked()
{
    if (startButton->text()=="Stop")
    {
        switchTimer->stop();
        qDebug()<<"Scrolling stopted";
        startButton->setText("Start");
    }
    else
    {
        if (scroll_label->text().contains("minutes")) switch_time=timeBox->text().toInt()*60*1000;
        else switch_time=timeBox->text().toInt()*60*60*1000;
        //switch_time=1000; //for debug
        switchTimer->start(switch_time);
        if (switchTimer->isActive()) qDebug()<<"Scrolling stagted";
        startButton->setText("Stop");
        set_image();
    }

    config_save();

    #ifdef MY_DEBUG
        qDebug().nospace()<<"Switch time: "<<switch_time/(1000*60)<<"min.";
    #endif
}

QList<int> WallWindow::create_randlist()
{
    QList<int> temp1, temp2;
    int rand_num;
    qsrand(QDateTime::currentDateTime().toTime_t());

    for (int i=0; i<allFiles.count(); ++i) temp1<<i;
    linear_list=temp1;

    for (int i=0; i<allFiles.count(); ++i)
    {
        rand_num=qrand()%temp1.count();
        temp2<<temp1.at(rand_num);
        temp1.removeAt(rand_num);
    }
    return temp2;
}

int WallWindow::set_image()
{
    //int number;
    if (row_num!=0)
    {
        number=row_num;
        order_comboBox_currentIndexChanged("Linear");
    }
    else number=rand_numbers.first();

    if (order_comboBox->currentText()=="Linear" && row_num==0)
    {
        number=linear_list.first();
        qDebug()<<"Set image:"<<allFiles.at(number).absoluteFilePath();
        processStart("/desktop/gnome/background/picture_filename",
                     allFiles.at(number).absoluteFilePath());
        linear_list.removeFirst();
    }
    else
    {
        qDebug()<<"Set image:"<<allFiles.at(number).absoluteFilePath();
        processStart("/desktop/gnome/background/picture_filename",
                     allFiles.at(number).absoluteFilePath());
        if (row_num==0) rand_numbers.removeFirst();
    }

    tableWidget->scrollToItem(tableWidget->item(number,1),QAbstractItemView::PositionAtTop);
    tableWidget->horizontalScrollBar()->setValue(0);
    trayIcon->setToolTip("Now: "+allFiles.at(number).fileName());
    if (rand_numbers.count()==0) rand_numbers=create_randlist();
    if (linear_list.count()==0) create_randlist();
    row_num=0;

    #ifdef MY_DEBUG
        qDebug()<<"Image number:"<<number;

        QDateTime dt = QDateTime();
        qDebug()<<dt.currentDateTime();
    #endif

    return 0;
}

void WallWindow::config_save()
{
    if (onlynames_checkBox->checkState()==2) settings->setValue("settings/show_only_names", true);
    else settings->setValue("settings/show_only_names", false);

    settings->setValue("settings/scroll_time", timeBox->value());

    if (scroll_label->text().contains("minutes")) settings->setValue("settings/in", "Scroll time(minutes):");
    else settings->setValue("settings/in", "Scroll time(hour):");

    settings->setValue("settings/img_position", pos_comboBox->currentIndex());

    settings->setValue("settings/img_order", order_comboBox->currentIndex());

    QStringList temp;
    for (int i=0; i<allFiles.count(); i++) temp.append(allFiles.at(i).absoluteFilePath());
    settings->setValue("settings/file_list", temp);
}

void WallWindow::on_onlynames_checkBox_stateChanged(int x)
{
    for(int i=0; i<allFiles.count(); i++)
    {
        QTableWidgetItem* Item = new QTableWidgetItem();
        if (x==2) Item->setText(QString(allFiles.at(i).fileName()));
        if (x==0) Item->setText(QString(allFiles.at(i).absoluteFilePath()));
        tableWidget->setItem(i, 1, Item);
    }
}

void WallWindow::on_timeBox_valueChanged(int time)
{
    if (time>59 && scroll_label->text().contains("minutes"))
    {
        timeBox->setValue(1);
        timeBox->setMaximum(24);
        timeBox->setMinimum(0);
        scroll_label->setText("Scroll time(hour):");
    }
    if (time==0)
    {
        timeBox->setMaximum(60);
        timeBox->setMinimum(1);
        timeBox->setValue(59);
        scroll_label->setText("Scroll time(minutes):");
    }
}

void WallWindow::add_clicked()
{
    QStringList added_files = QFileDialog::getOpenFileNames(this,
                                         "Open Image(s)",
                                          QDir::homePath (),
                                         "Image Files (*.png *.jpg *.bmp *.tiff *.tif)");

    for (int i=0; i<added_files.count(); i++)
    {
        QFileInfo info(added_files[i]);
        allFiles.append(info);
    }

    tableFill();
    tableWidget->resizeColumnsToContents();
    rand_numbers=create_randlist();
    config_save();
}

void WallWindow::on_tableWidget_cellDoubleClicked(int row, int column)
{
    row_num=row;
    if (column==1) set_image();
}

void WallWindow::on_pos_comboBox_currentIndexChanged(QString name)
{
    processStart("/desktop/gnome/background/picture_options",name.toLower());
}

void WallWindow::order_comboBox_currentIndexChanged(QString name)
{
    create_randlist();
    if (name=="Linear")
        for (int i=linear_list.first(); i<number+1; i++) linear_list.removeFirst();
}

Button::Button(const QString &text, QWidget *parent):QToolButton(parent){setText(text);}

void WallWindow::digitClicked()
{
    //Button *clickedButton = qobject_cast<Button *>(sender());
    //int nowClicked=clickedButton->accessibleName().toInt();

    //allFiles.removeAt(nowClicked);
    //tableWidget->removeRow(nowClicked);
    //qDebug()<<removeButtons.count()<<nowClicked;
    //removeButtons.removeAt(nowClicked);
    //qDebug()<<removeButtons.count();
    countlabel->setText("Images count: "+QString::number(allFiles.count()));
    tableWidget->resizeColumnsToContents();
    config_save();
}

Button *WallWindow::createButton(const QString &text, const char *member, int &num)
{
    Button *button = new Button(text);
    button->setAccessibleName(QString::number(num));
    connect(button, SIGNAL(clicked()), this, member);
    return button;
}

bool ispause=false;

void WallWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    if (reason==3 || reason==2)
    {
        qDebug()<<"Fast swich";
        set_image();
    }
    if (reason==4)
    {
        if (ispause==true)
        {
            start_clicked();
            qDebug()<<"Scrolling stagted";
            ispause=false;
        }
        else
        {
            startButton->setText("Start");
            switchTimer->stop();
            ispause=true;
            trayIcon->setToolTip("Scrolling stopted");
            qDebug()<<"Scrolling stopted";
        }
    }
}

void WallWindow::closeEvent(QCloseEvent * event)
{
    hide();
    event->ignore();
    config_save();
}

void WallWindow::quitButton_clicked()
{
    config_save();
    exit(0);
}

void WallWindow::processStart(QString path, QString arg)
{
    QProcess myProcess;
    QString program = "gconftool-2";
    QStringList arguments;
    arguments<<"--type"<<"string"<<"--set"<<path<<arg;
    myProcess.start(program, arguments);
    myProcess.waitForFinished();
}

void WallWindow::clear_clicked()
{
    allFiles.clear();
    tableWidget->clear();
    tableWidget->resizeColumnsToContents();
    tableWidget->setRowCount(0);
    tableWidget->setColumnCount(0);
    config_save();
}

void WallWindow::on_tableWidget_cellClicked(int row, int column)
{
    if (column==0)
    {
        tableWidget->removeRow(row);
        allFiles.removeAt(row);
        config_save();
    }
}
